@@ -3,7 +3,7 @@ pg_activity
 author: Julien Tachoires <julmon@gmail.com>
 license: PostgreSQL License
 
-Copyright (c) 2012 - 2016, Julien Tachoires
+Copyright (c) 2012 - 2019, Julien Tachoires
 
 Permission to use, copy, modify, and distribute this software and its
 documentation for any purpose, without fee, and without a written
@@ -29,6 +29,7 @@ import psutil
 import time
 from pgactivity.Process import Process
 import os
+from warnings import catch_warnings, simplefilter
 
 if psutil.version_info < (2, 0, 0):
     class PSProcess(psutil.Process):
@@ -112,7 +113,8 @@ class Data:
         user = 'postgres',
         password = None,
         database = 'postgres',
-        rds_mode = False):
+        rds_mode = False,
+        service = None):
         """
         Connect to a PostgreSQL server and return
         cursor & connector.
@@ -121,24 +123,36 @@ class Data:
         if host is None or host == 'localhost':
             # try to connect using UNIX socket
             try:
-                self.pg_conn = psycopg2.connect(
-                    database = database,
-                    user = user,
-                    port = port,
-                    password = password,
-                    connection_factory = psycopg2.extras.DictConnection
+                if service is not None:
+                    self.pg_conn = psycopg2.connect(
+                        service = service,
+                        connection_factory = psycopg2.extras.DictConnection
+                    )
+                else:
+                    self.pg_conn = psycopg2.connect(
+                        database = database,
+                        user = user,
+                        port = port,
+                        password = password,
+                        connection_factory = psycopg2.extras.DictConnection
                     )
             except psycopg2.Error as psy_err:
                 if host is None:
                     raise psy_err
         if self.pg_conn is None: # fallback on TCP/IP connection
-            self.pg_conn = psycopg2.connect(
-                database = database,
-                host = host,
-                port = port,
-                user = user,
-                password = password,
-                connection_factory = psycopg2.extras.DictConnection
+            if service is not None:
+                self.pg_conn = psycopg2.connect(
+                    service = service,
+                    connection_factory = psycopg2.extras.DictConnection
+                )
+            else:
+                self.pg_conn = psycopg2.connect(
+                    database = database,
+                    host = host,
+                    port = port,
+                    user = user,
+                    password = password,
+                    connection_factory = psycopg2.extras.DictConnection
                 )
         self.pg_conn.set_isolation_level(0)
         if rds_mode != True: # Make sure we are using superuser if not on RDS
@@ -150,7 +164,7 @@ class Data:
 
     def pg_is_local_access(self,):
         """
-        Verify if the user running pg_activity can acces 
+        Verify if the user running pg_activity can acces
         system informations for the postmaster process.
         """
         try:
@@ -233,7 +247,7 @@ class Data:
     def pg_get_num_dev_version(self, text_version):
         """
         Get PostgreSQL short & numeric devel. or beta version
-        from a string (SELECT version()). 
+        from a string (SELECT version()).
         """
         res = re.match(
             r"^(PostgreSQL|EnterpriseDB) ([0-9]+)(?:\.([0-9]+))?(devel|beta[0-9]+|rc[0-9]+)",
@@ -252,7 +266,7 @@ class Data:
             return
         raise Exception('Undefined PostgreSQL version.')
 
-    def pg_get_db_info(self, prev_db_infos, using_rds = False):
+    def pg_get_db_info(self, prev_db_infos, using_rds=False, skip_sizes=False):
         """
         Get current sum of transactions, total size and  timestamp.
         """
@@ -260,12 +274,15 @@ class Data:
     SELECT
         EXTRACT(EPOCH FROM NOW()) AS timestamp,
         SUM(pg_stat_get_db_xact_commit(oid)+pg_stat_get_db_xact_rollback(oid))::BIGINT AS no_xact,
-        SUM(pg_database_size(datname)) AS total_size,
+        {db_size} AS total_size,
         MAX(LENGTH(datname)) AS max_length
     FROM
         pg_database
-        """
-        query += "\nWHERE datname <> 'rdsadmin'" if using_rds else ''
+        {no_rds}
+        """.format(
+            db_size="0" if skip_sizes else "SUM(pg_database_size(datname))",
+            no_rds="WHERE datname <> 'rdsadmin'" if using_rds else ''
+        )
         cur = self.pg_conn.cursor()
         cur.execute(query,)
         ret = cur.fetchone()
@@ -285,6 +302,23 @@ class Data:
             'tps': tps,
             'size_ev': size_ev}
 
+    def pg_get_active_connections(self,):
+        """
+        Get total of active connections.
+        """
+        query = """
+        SELECT
+            COUNT(*) as active_connections
+        FROM pg_stat_activity
+        WHERE state = 'active'
+        """
+
+        cur = self.pg_conn.cursor()
+        cur.execute(query,)
+        ret = cur.fetchone()
+        active_connections = int(ret['active_connections'])
+        return active_connections
+
     def pg_get_activities(self,):
         """
         Get activity from pg_stat_activity view.
@@ -294,14 +328,19 @@ class Data:
             query = """
     SELECT
         pg_stat_activity.pid AS pid,
+        pg_stat_activity.application_name AS application_name,
         CASE WHEN LENGTH(pg_stat_activity.datname) > 16
             THEN SUBSTRING(pg_stat_activity.datname FROM 0 FOR 6)||'...'||SUBSTRING(pg_stat_activity.datname FROM '........$')
             ELSE pg_stat_activity.datname
             END
         AS database,
-        pg_stat_activity.client_addr AS client,
+        CASE WHEN pg_stat_activity.client_addr IS NULL
+            THEN 'local'
+            ELSE pg_stat_activity.client_addr::TEXT
+            END
+        AS client,
         EXTRACT(epoch FROM (NOW() - pg_stat_activity.query_start)) AS duration,
-        pg_stat_activity.wait_event IS NOT NULL AS wait,
+        CASE WHEN pg_stat_activity.wait_event_type IN ('LWLock', 'Lock', 'BufferPin') THEN true ELSE false END AS wait,
         pg_stat_activity.usename AS user,
         pg_stat_activity.state AS state,
         pg_stat_activity.query AS query,
@@ -315,16 +354,21 @@ class Data:
         EXTRACT(epoch FROM (NOW() - pg_stat_activity.query_start)) DESC
             """
         elif self.pg_num_version >= 90600:
-            # PostgreSQL 9.6.0 and more
+            # PostgreSQL prior to 10.0 and >= 9.6.0
             query = """
     SELECT
         pg_stat_activity.pid AS pid,
+        pg_stat_activity.application_name AS application_name,
         CASE WHEN LENGTH(pg_stat_activity.datname) > 16
             THEN SUBSTRING(pg_stat_activity.datname FROM 0 FOR 6)||'...'||SUBSTRING(pg_stat_activity.datname FROM '........$')
             ELSE pg_stat_activity.datname
             END
         AS database,
-        pg_stat_activity.client_addr AS client,
+        CASE WHEN pg_stat_activity.client_addr IS NULL
+            THEN 'local'
+            ELSE pg_stat_activity.client_addr::TEXT
+            END
+        AS client,
         EXTRACT(epoch FROM (NOW() - pg_stat_activity.query_start)) AS duration,
         pg_stat_activity.wait_event IS NOT NULL AS wait,
         pg_stat_activity.usename AS user,
@@ -339,17 +383,22 @@ class Data:
     ORDER BY
         EXTRACT(epoch FROM (NOW() - pg_stat_activity.query_start)) DESC
             """
-        elif self.pg_num_version < 90600 and self.pg_num_version >= 90200:
+        elif self.pg_num_version >= 90200:
             # PostgreSQL prior to 9.6.0 and >= 9.2.0
             query = """
     SELECT
         pg_stat_activity.pid AS pid,
+        pg_stat_activity.application_name AS application_name,
         CASE WHEN LENGTH(pg_stat_activity.datname) > 16
             THEN SUBSTRING(pg_stat_activity.datname FROM 0 FOR 6)||'...'||SUBSTRING(pg_stat_activity.datname FROM '........$')
             ELSE pg_stat_activity.datname
             END
         AS database,
-        pg_stat_activity.client_addr AS client,
+        CASE WHEN pg_stat_activity.client_addr IS NULL
+            THEN 'local'
+            ELSE pg_stat_activity.client_addr::TEXT
+            END
+        AS client,
         EXTRACT(epoch FROM (NOW() - pg_stat_activity.query_start)) AS duration,
         pg_stat_activity.waiting AS wait,
         pg_stat_activity.usename AS user,
@@ -369,13 +418,18 @@ class Data:
             query = """
     SELECT
         pg_stat_activity.procpid AS pid,
+        '<unknown>' AS application_name,
         CASE
             WHEN LENGTH(pg_stat_activity.datname) > 16
             THEN SUBSTRING(pg_stat_activity.datname FROM 0 FOR 6)||'...'||SUBSTRING(pg_stat_activity.datname FROM '........$')
             ELSE pg_stat_activity.datname
             END
         AS database,
-        pg_stat_activity.client_addr AS client,
+        CASE WHEN pg_stat_activity.client_addr IS NULL
+            THEN 'local'
+            ELSE pg_stat_activity.client_addr::TEXT
+            END
+        AS client,
         EXTRACT(epoch FROM (NOW() - pg_stat_activity.query_start)) AS duration,
         pg_stat_activity.waiting AS wait,
         pg_stat_activity.usename AS user,
@@ -402,6 +456,7 @@ class Data:
             query = """
     SELECT
         pg_locks.pid AS pid,
+        pg_stat_activity.application_name AS appname,
         CASE WHEN LENGTH(pg_stat_activity.datname) > 16
             THEN SUBSTRING(pg_stat_activity.datname FROM 0 FOR 6)||'...'||SUBSTRING(pg_stat_activity.datname FROM '........$')
             ELSE pg_stat_activity.datname
@@ -427,6 +482,7 @@ class Data:
             query = """
     SELECT
         pg_locks.pid AS pid,
+        '<unknown>' AS appname,
         CASE
             WHEN LENGTH(pg_stat_activity.datname) > 16
             THEN SUBSTRING(pg_stat_activity.datname FROM 0 FOR 6)||'...'||SUBSTRING(pg_stat_activity.datname FROM '........$')
@@ -438,7 +494,7 @@ class Data:
         pg_locks.locktype AS type,
         pg_locks.relation::regclass AS relation,
         EXTRACT(epoch FROM (NOW() - pg_stat_activity.query_start)) AS duration,
-        pg_stat_activity.state as state,
+        NULL AS state,
         pg_stat_activity.current_query AS query
     FROM
         pg_catalog.pg_locks
@@ -462,6 +518,7 @@ class Data:
             query = """
     SELECT
         pid,
+        application_name AS appname,
         CASE
             WHEN LENGTH(datname) > 16
             THEN SUBSTRING(datname FROM 0 FOR 6)||'...'||SUBSTRING(datname FROM '........$')
@@ -479,6 +536,7 @@ class Data:
         (
         SELECT
             blocking.pid,
+            pg_stat_activity.application_name,
             pg_stat_activity.query,
             blocking.mode,
             pg_stat_activity.datname,
@@ -502,6 +560,7 @@ class Data:
         UNION ALL
         SELECT
             blocking.pid,
+            pg_stat_activity.application_name,
             pg_stat_activity.query,
             blocking.mode,
             pg_stat_activity.datname,
@@ -528,6 +587,7 @@ class Data:
         ) AS sq
     GROUP BY
         pid,
+        application_name,
         query,
         mode,
         locktype,
@@ -556,16 +616,17 @@ class Data:
         duration,
         state,
         query
-    FROM 
+    FROM
         (
         SELECT
             blocking.pid,
+            '<unknown>' AS appname,
             pg_stat_activity.current_query AS query,
             blocking.mode,
             pg_stat_activity.datname,
             pg_stat_activity.usename,
             blocking.locktype,EXTRACT(epoch FROM (NOW() - pg_stat_activity.query_start)) AS duration,
-            pg_stat_activity.state as state,
+            NULL AS state,
             blocking.relation::regclass AS relation
         FROM
             pg_locks AS blocking
@@ -582,13 +643,14 @@ class Data:
         UNION ALL
         SELECT
             blocking.pid,
+            '<unknown>' AS appname,
             pg_stat_activity.current_query AS query,
             blocking.mode,
             pg_stat_activity.datname,
             pg_stat_activity.usename,
             blocking.locktype,
             EXTRACT(epoch FROM (NOW() - pg_stat_activity.query_start)) AS duration,
-            pg_stat_activity.state as state,
+            NULL AS state,
             blocking.relation::regclass AS relation
         FROM
             pg_locks AS blocking
@@ -608,6 +670,7 @@ class Data:
         ) AS sq
     GROUP BY
         pid,
+        appname,
         query,
         mode,
         locktype,
@@ -675,8 +738,9 @@ class Data:
                     duration = query['duration'],
                     wait = query['wait'],
                     state = query['state'],
-                    query = clean_str(query['query']),
-                    extras = {}
+                    query = query['query'],
+                    extras = {},
+                    appname = query['application_name']
                     )
 
                 process.set_extra('meminfo',
@@ -697,6 +761,8 @@ class Data:
                     self.__sys_get_iow_status(psproc.status_iow()))
                 process.set_extra('psutil_proc', psproc)
                 process.set_extra('backend_type', query['backend_type'])
+                process.set_extra('appname', query['application_name'])
+
                 processes[process.pid] = process
 
             except psutil.NoSuchProcess:
@@ -732,18 +798,20 @@ class Data:
         """
         Get memory and swap usage
         """
-        try:
-            # psutil >= 0.6.0
-            phymem = psutil.virtual_memory()
-            buffers = psutil.virtual_memory().buffers
-            cached = psutil.virtual_memory().cached
-            vmem = psutil.swap_memory()
-        except AttributeError:
-            # psutil > 0.4.0 and < 0.6.0
-            phymem = psutil.phymem_usage()
-            buffers = getattr(psutil, 'phymem_buffers', lambda: 0)()
-            cached = getattr(psutil, 'cached_phymem', lambda: 0)()
-            vmem = psutil.virtmem_usage()
+        with catch_warnings():
+            simplefilter("ignore", RuntimeWarning)
+            try:
+                # psutil >= 0.6.0
+                phymem = psutil.virtual_memory()
+                buffers = psutil.virtual_memory().buffers
+                cached = psutil.virtual_memory().cached
+                vmem = psutil.swap_memory()
+            except AttributeError:
+                # psutil > 0.4.0 and < 0.6.0
+                phymem = psutil.phymem_usage()
+                buffers = getattr(psutil, 'phymem_buffers', lambda: 0)()
+                cached = getattr(psutil, 'cached_phymem', lambda: 0)()
+                vmem = psutil.virtmem_usage()
 
         mem_used = phymem.total - (phymem.free + buffers + cached)
         return (
